@@ -136,8 +136,10 @@ def centroid_box_iou(box1, box2):
 
 def to_centroid(minmax_boxes):
     """
-    minmax_boxes : (N, 4)
+    minmax_boxes : (N, 4) [[100, 120, 140, 200]]
+    centroid_boxes: [[120. 160.  40.  80.]]
     """
+    #minmax_boxes = np.asarray([[100, 120, 140, 200]])
     minmax_boxes = minmax_boxes.astype(np.float)
     centroid_boxes = np.zeros_like(minmax_boxes)
     
